      program test_sgemm_main
      implicit none
      integer, parameter :: nnn = 4096, lda = 4096, itaration = 1

      call test_sgemm_sub(nnn, lda, itaration)
      endprogram test_sgemm_main
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine test_sgemm_sub(nnn, lda, itaration)
      implicit none
      integer, intent(in) :: nnn, lda, itaration
      integer :: i, j
      real :: aaa(lda,nnn), bbb(lda,nnn), ccc1(lda,nnn), ccc2(lda,nnn)
      real, parameter :: alpha = 3.141592, beta = 0.001
      real(8), external :: second
      real(8) :: time1, time2, time3, maxerror, tmperror
      real(8) :: flops1, flops2

      ! Initialize array
      do j = 1, nnn
         do i = 1, nnn
            aaa(i,j) = float(i) / float(j)
            bbb(i,j) = sqrt(float(i+j))
            ccc1(i,j) = float(j) / float(i)
            ccc2(i,j) = float(j) / float(i)
         enddo
      enddo

      ! Invoke sgemm
      time1 = second() * 1.0d-6
      do i = 1, itaration
         call sgemm('N', 'N', nnn, nnn, nnn, alpha,
     $        aaa, lda, bbb, lda, beta, ccc1, lda)
      enddo
      time2 = second() * 1.0d-6
      do i = 1, itaration
         call my_sgemm('N', 'N', nnn, nnn, nnn, alpha,
     $        aaa, lda, bbb, lda, beta, ccc2, lda)
      enddo
      time3 = second() * 1.0d-6

      ! Calculate flops
      flops1 = dble(nnn)**3 * 2.0d+0 * dble(itaration) / (time2 - time1)
      flops2 = dble(nnn)**3 * 2.0d+0 * dble(itaration) / (time3 - time2)

      ! Calculate error
      maxerror = 0.0d+0
      do j = 1, nnn
         do i = 1, nnn
            tmperror = abs((ccc1(i,j) - ccc2(i,j)) / ccc1(i,j))
            maxerror = max(maxerror, tmperror)
         enddo
      enddo

      ! Print
      write(*,'("array size  : ", i5)') nnn
      write(*,'("leading dim : ", i5)') lda
      write(*,'("itaration   : ", i5)') itaration
      write(*,'("sgemm       :", 1pg15.7, " MFLOPS")') flops1 * 1.0d-6
      write(*,'("my_sgemm    :", 1pg15.7, " MFLOPS")') flops2 * 1.0d-6
      write(*,'("rel. error  :", 1pg15.7)') maxerror

      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      function second() result(rtime)
      implicit none
      integer :: ic,ir,im
      real(8) :: rtime

      call system_clock(ic,ir,im)
      rtime= real(ic,8)
      end function second
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
